package sialocacao;

/**
 * <p>Title: Database.java </p>
 * <p>Description: Responsvel pelo controle de conexo com o banco de dados. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 2.0
 */

import java.sql.*;

public class Instalador {

  private Connection conexao;

  public Instalador(Connection conexao) {
    if (conexao == null) {
      throw new IllegalArgumentException("conexao == null");
    }
    this.conexao = conexao;
  }

  /**
   * Retorna a conexo corrente.
   * @return conexao
   */
  public Connection getConexao() {
    return conexao;
  }

  /**
   * Cria tabelas no banco de dados.
   */
  public void criaTabelas() throws SQLException {
    Statement statement = null;
    StringBuffer query = new StringBuffer();

    try {
      conexao.setAutoCommit(false);
      statement = conexao.createStatement();

      /**
       * Criando sequencia da tabela professor.
       */
      query.setLength(0);
      query.append("create sequence CODIGO_PROFESSOR");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando tabela de professor.
       */
      query.setLength(0);
      query.append("CREATE TABLE professor(");
      query.append("codigo SERIAL PRIMARY KEY, ");
      query.append("nome VARCHAR(80) UNIQUE, ");
      query.append("cpf VARCHAR(15) UNIQUE NOT NULL");
      query.append(")");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando sequencia da tabela horario.
       */
      query.setLength(0);
      query.append("create sequence CODIGO_HORARIO ");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando tabela de horario.
       */
      query.setLength(0);
      query.append("CREATE TABLE horario(");
      query.append("codigo SERIAL PRIMARY KEY, ");
      query.append("diaDaSemana VARCHAR(15) NOT NULL, ");
      query.append("horaInicio VARCHAR(6) NOT NULL, ");
      query.append("horaFim VARCHAR(6) NOT NULL, ");
      query.append("UNIQUE (diaDaSemana, horaInicio, horaFim)");
      query.append(")");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando sequencia da tabela categoria.
       */
      query.setLength(0);
      query.append("create sequence CODIGO_CATEGORIA ");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando tabela de categoria.
       */
      query.setLength(0);
      query.append("CREATE TABLE categoria(");
      query.append("codigo SERIAL PRIMARY KEY, ");
      query.append("descricao VARCHAR(150) UNIQUE");
      query.append(")");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando sequencia da tabela sala.
       */
      query.setLength(0);
      query.append("create sequence CODIGO_SALA");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando tabela de sala.
       */
      query.setLength(0);
      query.append("CREATE TABLE sala(");
      query.append("codigo SERIAL PRIMARY KEY, ");
      query.append("descricao VARCHAR(80) UNIQUE, ");
      query.append("localizacao VARCHAR(80), ");
      query.append("observacao VARCHAR(80), ");
      query.append("codigoCategoria INTEGER NOT NULL, ");
      query.append("FOREIGN KEY (codigoCategoria) REFERENCES categoria (codigo)");
      query.append(")");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando sequencia da tabela disciplina.
       */
      query.setLength(0);
      query.append("create sequence CODIGO_DISCIPLINA ");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando tabela de diciplina.
       */
      query.setLength(0);
      query.append("CREATE TABLE disciplina(");
      query.append("codigo SERIAL PRIMARY KEY, ");
      query.append("descricao VARCHAR(150) UNIQUE, ");
      query.append("cargaHoraria INTEGER, ");
      query.append("ementa TEXT, ");
      query.append("codigoProfessor INTEGER, ");
      query.append("FOREIGN KEY (codigoProfessor) REFERENCES professor (codigo)");
      query.append(")");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando tabela de professor_horario.
       */
      query.setLength(0);
      query.append("CREATE TABLE professor_horario(");
      query.append("codigoProfessor INTEGER NOT NULL, ");
      query.append("codigoHorario INTEGER NOT NULL, ");
      query.append("prioridade INTEGER NOT NULL, ");
      query.append("UNIQUE (codigoProfessor, codigoHorario), ");
      query.append("FOREIGN KEY (codigoProfessor) REFERENCES professor (codigo), ");
      query.append("FOREIGN KEY (codigoHorario) REFERENCES horario (codigo)");
      query.append(")");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando tabela de disciplina_categoria.
       */
      query.setLength(0);
      query.append("CREATE TABLE disciplina_categoria(");
      query.append("codigoDisciplina INTEGER NOT NULL, ");
      query.append("codigoCategoria INTEGER NOT NULL, ");
      query.append("prioridade INTEGER NOT NULL, ");
      query.append("UNIQUE (codigoDisciplina, codigoCategoria), ");
      query.append("FOREIGN KEY (codigoDisciplina) REFERENCES disciplina (codigo), ");
      query.append("FOREIGN KEY (codigoCategoria) REFERENCES categoria (codigo)");
      query.append(")");
      System.out.println(query);
      statement.executeUpdate(query.toString());

      /**
       * Criando tabela de alocacao.
       */
      query.setLength(0);
      query.append("CREATE TABLE alocacao(");
      query.append("codigo SERIAL PRIMARY KEY, ");
      query.append("codigoSala INTEGER NOT NULL, ");
      query.append("codigoHorario INTEGER NOT NULL, ");
      query.append("codigoDisciplina INTEGER NOT NULL, ");
      query.append("codigoProfessor INTEGER NOT NULL, ");
      query.append("data VARCHAR(11) NOT NULL, ");
      query.append("UNIQUE (codigoSala, codigoHorario, data), ");
      query.append("UNIQUE(codigoProfessor, codigoHorario, data), ");
      query.append("FOREIGN KEY (codigoSala) REFERENCES sala (codigo), ");
      query.append("FOREIGN KEY (codigoProfessor) REFERENCES professor (codigo), ");
      query.append("FOREIGN KEY (codigoHorario) REFERENCES horario (codigo), ");
      query.append("FOREIGN KEY (codigoDisciplina) REFERENCES disciplina (codigo)");
      query.append(")");
      System.out.println(query);
      statement.executeUpdate(query.toString());

    } catch(SQLException exception0) {
      System.err.println("SQLException: " + exception0.getMessage()) ;
      try {
        conexao.rollback() ;
      } catch (SQLException exception1) {}
      throw exception0;
    } finally {
        if (statement != null) {
            try {
                statement.close();
            } catch (SQLException exception) {}
        }
    }
  }

  /**
   * Remove tabelas do banco de dados.
   */
  public void removeTabelas() throws SQLException {
    Statement statement = null;

    try {
      statement = conexao.createStatement();

      try {
          statement.executeUpdate("DROP TABLE alocacao");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("DROP TABLE disciplina_categoria");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("DROP TABLE professor_horario");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("DROP TABLE disciplina");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("DROP TABLE sala");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("DROP TABLE categoria");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("DROP TABLE horario");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("DROP TABLE professor");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("drop sequence CODIGO_PROFESSOR");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("drop sequence CODIGO_HORARIO");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("drop sequence CODIGO_DISCIPLINA");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("drop sequence CODIGO_CATEGORIA");
      } catch (SQLException exception) {}

      try {
          statement.executeUpdate("drop sequence CODIGO_SALA");
      } catch (SQLException exception) {}

    } finally {
        if (statement != null) {
            try {
                statement.close();
            } catch (SQLException exception) {}
        }
    }
  }

  /**
   * Popular base com horario.
   */
  public void inserirHorarios() throws SQLException {
    Statement statement = null;
    StringBuffer query = new StringBuffer();

    String[] diasDaSemana = {"Segunda", "Tera", "Quarta", "Quinta", "Sexta", "Sbado"};
    String[] horasInicio  = {"08:00", "09:00", "10:00", "11:00", "12:00",
                             "13:00", "14:00", "15:00", "16:00", "17:00"};
    String[] horasFim     = {"09:00", "10:00", "11:00", "12:00", "13:00",
                             "14:00", "15:00", "16:00", "17:00", "18:00"};
    int indexDia = 0;
    int indexInicio = 0;
    int indexFim = 0;

    try {
      statement = conexao.createStatement();

      for(indexInicio=0; indexInicio<horasInicio.length; indexInicio++) {
         for(indexDia=0; indexDia<diasDaSemana.length; indexDia++) {

            query.setLength(0);
            query.append("INSERT into HORARIO (");
            query.append("DIADASEMANA, ");
            query.append("HORAINICIO, ");
            query.append("HORAFIM) values ('");
            query.append(diasDaSemana[indexDia]);
            query.append("', '");
            query.append(horasInicio[indexInicio]);
            query.append("', '");
            query.append(horasFim[indexFim]);
            query.append("')");
            System.out.println(query);
            statement.executeUpdate(query.toString());

         }
         indexFim ++;
      }

    } catch(SQLException exception0) {
      System.err.println("SQLException: " + exception0.getMessage()) ;
      try {
        conexao.rollback() ;
      } catch (SQLException exception1) {}
      throw exception0;
    } finally {
        if (statement != null) {
            try {
                statement.close();
            } catch (SQLException exception) {}
        }
    }
  }

  /**
   * Popular base com categorias de sala.
   */
  public void inserirCategorias() throws SQLException {
    Statement statement = null;
    StringBuffer query = new StringBuffer();

    String[] categoriasDeSala = {"Esportivo", "Auditrio",
                                 "Sala com recursos audiovisuais",
                                 "Sala quadro branco", "Laboratrio"};
    int index = 0;

    try {
      statement = conexao.createStatement();

      for(index=0; index<categoriasDeSala.length; index++) {

         query.setLength(0);
         query.append("INSERT into CATEGORIA (");
         query.append("DESCRICAO) values ('");
         query.append(categoriasDeSala[index]);
         query.append("')");
         System.out.println(query);
         statement.executeUpdate(query.toString());
      }

    } catch(SQLException exception0) {
      System.err.println("SQLException: " + exception0.getMessage()) ;
      try {
        conexao.rollback() ;
      } catch (SQLException exception1) {}
      throw exception0;
    } finally {
        if (statement != null) {
            try {
                statement.close();
            } catch (SQLException exception) {}
        }
    }
  }

  public static void main(String[] args) {
    Connection con = null;

    // Posgress
    String driver = "org.postgresql.Driver";
    String url = "jdbc:postgresql://localhost/sialocacaoplus";

    // Access
//    String driver = "sun.jdbc.odbc.JdbcOdbcDriver";
//    String url = "jdbc:odbc:sialocacaoplus";

    try {
      Class.forName(driver);
      con = DriverManager.getConnection(
          url, "lecl", "C1s0Pr&");
      con.setAutoCommit(false);

      Instalador instalador = new Instalador(con);
      instalador.removeTabelas();
      instalador.criaTabelas();
      instalador.inserirHorarios();
      instalador.inserirCategorias();
      con.commit();
      con.setAutoCommit(true);
    } catch(Exception exception) {
      exception.printStackTrace();
    } finally {
        try {
          con.setAutoCommit(true);
        } catch (SQLException exception) {}
    }
  }
}