package sialocacao;

/**
 * <p>Title: SIAlocacao.java </p>
 * <p>Description: Responsvel pelo controle do sistema. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos Andreazza Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 2.0
 */

import java.util.*;

import sialocacao.professor.*;
import sialocacao.professor.excecoes.*;
import sialocacao.professor.negocio.*;

import sialocacao.disciplina.*;
import sialocacao.disciplina.excecoes.*;
import sialocacao.disciplina.negocio.*;

import sialocacao.sala.*;
import sialocacao.sala.excecoes.*;
import sialocacao.sala.negocio.*;

import sialocacao.alocacao.*;
import sialocacao.alocacao.excecoes.*;
import sialocacao.alocacao.negocio.*;
import sialocacao.util.*;


public class SIAlocacao {

  /*
   * Cadastro de profesores.
   */
  private ICadastroProfessores cadastroProfessores;

  /*
   * Cadastro de disciplinas.
   */
  private ICadastroDisciplinas cadastroDisciplinas;

  /*
   * Cadastro de salas.
   */
  private ICadastroSalas cadastroSalas;

  /*
   * Cadastro de categoria.
   */
  private ICadastroCategorias cadastroCategorias;


  /*
   * Interface para realizar de alocacoes.
   */
  private ICadastroAlocacoes cadastroAlocacoes;

  /*
   * Construtor default responsvel por instanciar os cadastros do sistema.
   */
  public SIAlocacao() {
    try {
      setCadastroProfessores(new CadastroProfessores());
      setCadastroDisciplinas(new CadastroDisciplinas());
      setCadastroSalas(new CadastroSalas());
      setCadastroCategorias(new CadastroCategorias());
      setCadastroAlocacoes(new CadastroAlocacoes());
    } catch(Exception e) {
      e.printStackTrace();
    }
  }

  /*
   * Construtor responsvel por instanciar os cadastros do sistema.
   */
  public SIAlocacao(ICadastroProfessores cadastroProfessores,
                    ICadastroDisciplinas cadastroDisciplinas,
                    ICadastroSalas cadastroSalas,
                    ICadastroCategorias cadastroCategorias,
                    ICadastroAlocacoes cadastroAlocacoes) {

    setCadastroProfessores(cadastroProfessores);
    setCadastroDisciplinas(cadastroDisciplinas);
    setCadastroSalas(cadastroSalas);
    setCadastroCategorias(cadastroCategorias);
    setCadastroAlocacoes(cadastroAlocacoes);
  }

  /**
   * Define cadastro de professores.
   * @param cadastroProfessores
   */
  public void setCadastroProfessores(ICadastroProfessores cadastroProfessores) throws
      IllegalArgumentException {

    if (cadastroProfessores == null) {
      throw new IllegalArgumentException("cadastroProfessores == null");
    }
    this.cadastroProfessores = cadastroProfessores;
  }

  /**
   * Define cadastro de disciplinas.
   * @param cadastroDisciplinas
   */
  public void setCadastroDisciplinas(ICadastroDisciplinas cadastroDisciplinas) throws
    IllegalArgumentException {

    if (cadastroDisciplinas == null) {
      throw new IllegalArgumentException("cadastroDisciplinas == null");
    }
    this.cadastroDisciplinas = cadastroDisciplinas;
  }

  /**
   * Define cadastro de salas.
   * @param cadastroSalas
   */
  public void setCadastroSalas(ICadastroSalas cadastroSalas) throws
      IllegalArgumentException {

    if (cadastroSalas == null) {
      throw new IllegalArgumentException("cadastroSalas == null");
    }
    this.cadastroSalas = cadastroSalas;
  }

  /**
     * Define cadastro de categorias.
     * @param cadastroCategorias
     */
    public void setCadastroCategorias(ICadastroCategorias cadastroCategorias) throws
        IllegalArgumentException {

      if (cadastroCategorias == null) {
        throw new IllegalArgumentException("cadastroCategorias == null");
      }
      this.cadastroCategorias = cadastroCategorias;
    }

  /**
   * Define cadastro de alocaes
   * @param cadastroAlocacoes
   */

  public void setCadastroAlocacoes(ICadastroAlocacoes cadastroAlocacoes) throws
      IllegalArgumentException {

    if (cadastroAlocacoes == null) {
      throw new IllegalArgumentException("cadastroAlocacoes == null");
    }
    this.cadastroAlocacoes = cadastroAlocacoes;
  }

/*****************************************************************************
 *** CadastroDeProfessores ***************************************************
 *****************************************************************************/

  /*
   * Insere professor no sistema.
   * @param nome
   * @param area
   */
  public void inserirProfessor(String nome, String cpf) throws
      SIAlocacaoException, ProfessorJaCadastradoException {

    this.cadastroProfessores.inserirProfessor(nome, cpf);
  }

  /*
   * Altera professor no sistema.
   * @param professor
   */
  public void alterarProfessor(Professor professor) throws SIAlocacaoException,
      ProfessorInexistenteException {

    this.cadastroProfessores.alterarProfessor(professor);
  }

  /*
   * Exclui professor do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirProfessor(int codigo) throws SIAlocacaoException,
      ProfessorInexistenteException {

    removeProfessorDisciplina(codigo);
    this.cadastroProfessores.excluirProfessor(codigo);
  }

  /*
   * Busca professor no sistema pelo cdigo.
   * @param codigo
   * @return Professor
   */
  public Professor buscarProfessor(int codigo) throws SIAlocacaoException {

    return this.cadastroProfessores.buscarProfessor(codigo);
  }

  /*
   * Busca professor de uma disciplina no sistema.
   * @param codigoDisciplina
   * @return Professor
   */
  public Professor buscarProfessorDeDisciplina(int codigoDisciplina) throws SIAlocacaoException {
   Professor professor = null;
   int codProfessor;

   codProfessor = cadastroDisciplinas.buscarProfessorDeDisciplina(codigoDisciplina);
   if(codProfessor != 0) {
      professor = cadastroProfessores.buscarProfessor(codProfessor);
   }
    return professor;
  }

  /*
   * Busca professor no sistema pelo cdigo.
   * @param nome
   * @return Professor
   */
  public Professor buscarProfessor(String nome) throws SIAlocacaoException {

    return this.cadastroProfessores.buscarProfessor(nome);
  }

  /*
   * Lista professores do sistema.
   * @return ArrayList
   */
  public ArrayList listarProfessores() throws SIAlocacaoException {

    return this.cadastroProfessores.listarProfessores();
  }

  /*
   * Insere os horarios  preferenciais do professor no sistema.
   * @param codigoProfessor
   * @param codigoHorario
   * @param preferencia
   */
  public void inserirHorariosProfessor(int codigoProfessor, int codigoHorario,
      int prioridade) throws SIAlocacaoException {

    this.cadastroProfessores.inserirHorariosProfessor(codigoProfessor,
                                                      codigoHorario,prioridade);
  }

  /*
   * Altera os horarios  preferenciais do professor no sistema.
   * @param codigoProfessor
   * @param codigoHorario
   * @param preferencia
   */
  public void alterarHorariosProfessor(int codigoProfessor, int codigoHorario,
      int prioridade) throws SIAlocacaoException {

    this.cadastroProfessores.alterarHorariosProfessor(codigoProfessor,
                                                      codigoHorario,prioridade);
  }

  public Map buscarPrioridades(Professor professor) throws SIAlocacaoException{
    return this.cadastroProfessores.buscarPrioridades(professor);
  }

  public ArrayList listarPreferencias(int codProfessor)
     throws SIAlocacaoException{

     return this.cadastroProfessores.listarPreferencias(codProfessor);
  }

/*****************************************************************************
 *** CadastroDeDisciplinas ***************************************************
 *****************************************************************************/

  /*
   * Insere disciplina no sistema.
   * @param descricao
   * @param cargaHoraria
   * @param ementa
   */
  public void inserirDisciplina(String descricao, int cargaHoraria, String ementa, Map preferencias)
      throws SIAlocacaoException {

    this.cadastroDisciplinas.inserirDisciplina(descricao, cargaHoraria, ementa, preferencias);
  }

  /*
   * Altera disciplina no sistema.
   * @param disciplina
   */
  public void alterarDisciplina(Disciplina disciplina, Map prioridades)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    this.cadastroDisciplinas.alterarDisciplina(disciplina, prioridades);
  }

  /*
   * Exclui disciplina do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    this.cadastroDisciplinas.excluirDisciplina(codigo);
  }

  /*
   * Busca disciplina no sistema pelo cdigo.
   * @param codigo
   * @return Disciplina
   */
  public Disciplina buscarDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    return this.cadastroDisciplinas.buscarDisciplina(codigo);
  }

  /*
   * Busca disciplina no sistema pela descricao.
   * @param descricao
   * @return Disciplina
   */
  public Disciplina buscarDisciplina(String descricao)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    return this.cadastroDisciplinas.buscarDisciplina(descricao);
  }

  /*
   * Lista disciplinas do sistema.
   * @return ArrayList
   */
  public ArrayList listarDisciplinas() throws
      SIAlocacaoException, DisciplinaInexistenteException {

    return this.cadastroDisciplinas.listarDisciplinas();
  }


  public Map buscarPrioridades(Disciplina disciplina, List categorias) throws
      SIAlocacaoException {
    return this.cadastroDisciplinas.buscarPrioridades(disciplina, categorias);
  }

  /*
   * Associa disciplina a professor no sistema.
   * @param disciplina
   * @param codProfessor
   */
  public void associaProfessorDisciplina(Disciplina disciplina, int codProfessor)
      throws SIAlocacaoException, DisciplinaInexistenteException {
    this.cadastroDisciplinas.associaProfessorDisciplina(disciplina, codProfessor);
  }

  /*
   * Remove associacao de disciplina a professor no sistema.
   * @param codProfessor
   */
  public void removeProfessorDisciplina(int codProfessor) throws
       SIAlocacaoException, DisciplinaInexistenteException {
    this.cadastroDisciplinas.removeProfessorDisciplina(codProfessor);
  }


  /*
   * Verifica se todas as disciplinas estao associadas a um professor
   * @return true
   */
  public boolean todasDisciplinasAssociadas()
      throws SIAlocacaoException{

    return this.cadastroDisciplinas.todasDisciplinasAssociadas();
  }

/*****************************************************************************
 *** CadastroDeSalas *********************************************************
 *****************************************************************************/

  /*
   * Insere sala no sistema.
   * @param descricao
   * @param localizacao
   * @param observacao
   */
  public void inserirSala(String descricao, String localizacao, String observacao,
    int codigoCategoria)
      throws SIAlocacaoException {

    this.cadastroSalas.inserirSala(descricao, localizacao, observacao,
      codigoCategoria);
  }

  /*
   * Altera sala no sistema.
   * @param sala
   */
  public void alterarSala(Sala sala) throws SIAlocacaoException,
      SalaInexistenteException {

    this.cadastroSalas.alterarSala(sala);
  }

  /*
   * Exclui sala do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirSala(int codigo) throws SIAlocacaoException,
      SalaInexistenteException {

    this.cadastroSalas.excluirSala(codigo);
  }

  /*
   * Busca sala no sistema pelo cdigo.
   * @param codigo
   * @return Sala
   */
  public Sala buscarSala(int codigo) throws SIAlocacaoException {

    return this.cadastroSalas.buscarSala(codigo);
  }

  /*
   * Busca sala no sistema pelo cdigo.
   * @param descricao
   * @return Sala
   */
  public Sala buscarSala(String descricao) throws SIAlocacaoException {

    return this.cadastroSalas.buscarSala(descricao);
  }

  /*
   * Lista salas do sistema.
   * @return ArrayList
   */
  public ArrayList listarSalas() throws SIAlocacaoException {

    return this.cadastroSalas.listarSalas();
  }


  /*****************************************************************************
   *** CadastroDeCategorias *********************************************************
   *****************************************************************************/
  /*
   * Lista categorias do sistema.
   * @return ArrayList
   */
  public ArrayList listarCategorias() throws SIAlocacaoException {

    return this.cadastroCategorias.listarCategorias();
  }



/******************************************************************************
 *** CadastraAlocacao *********************************************************
 ******************************************************************************/

 /*
   * Insere uma Alocacao no sistema.
   * @param horario
   * @param sala
   * @param disciplina
   * @param professor
   */
  public void alocar(Sala sala, Disciplina disciplina, Horario horario, Professor professor, Data data)
       throws SIAlocacaoException{
    this.cadastroAlocacoes.alocar(sala, disciplina, horario, professor, data);
  }

  /**
   * Retorna um objeto disciplina contendo as informaoes da disciplina alocada
   * no horario e sala especificados.
   * @return sala
   */
  public Disciplina getDisciplinaAlocada(Sala sala, Horario horario, Data data) throws SIAlocacaoException{
    return this.cadastroAlocacoes.getDisciplinaAlocada(sala, horario, data);
  }

  /**
   * Exclui Alocacao do sistema.
   * @param alocacao
   */
  public void excluirAlocacao(int codigo) throws SIAlocacaoException{
    this.cadastroAlocacoes.excluirAlocacao(codigo);
  }

  /**
   * Altera Alocacao no sistema.
   * @param alocacao
   */
  public void alterarAlocacao(Alocacao alocacao) throws SIAlocacaoException {
    this.cadastroAlocacoes.alterarAlocacao(alocacao);
  }

  /**
   * Lista alocacoes do sistema.
   * @return ArrayList
   */
  public ArrayList listarAlocacoes() throws SIAlocacaoException{
    return this.cadastroAlocacoes.listarAlocacoes();
  }


  /**
   * Retorna o objeto alocacao para horario e sala especificados.
   * @param horario
   * @return sala
   */
  public Alocacao getAlocacao(Sala sala, Horario horario, Data data) throws SIAlocacaoException {
    return this.cadastroAlocacoes.getAlocacao(sala, horario, data);
  }


  /**
   * Lista horarios do sistema.
   * @return ArrayList
   */
  public ArrayList listarHorarios() throws SIAlocacaoException {

    return this.cadastroAlocacoes.listarHorarios();
  }

  /*
   * Retorna um objeto horario correspondente aos parametros passados
   * @param diadaSemana
   * @param horaInicio
   * @param horaFim
   * @return Horario
   */
  public Horario buscarHorario(String diaDaSemana, String horaInicio, String horaFim)
                               throws SIAlocacaoException {
    return this.cadastroAlocacoes.buscarHorario(diaDaSemana, horaInicio, horaFim);
  }

  /*
   * Retorna um objeto horario pelo codigo
   * @param codigoHorario
   * @return Horario
   */
  public Horario buscarHorario(int codigoHorario)
                               throws SIAlocacaoException {
    return this.cadastroAlocacoes.buscarHorario(codigoHorario);
  }
}