package sialocacao.alocacao;
/**
 * &lt;p&gt;Title: Alocacao.java &lt;/p&gt;
 * &lt;p&gt;Description: Responsvel por manipular o objeto alocacao. &lt;/p&gt;
 * &lt;p&gt;Copyright: Copyright (c) 2003&lt;/p&gt;
 * &lt;p&gt;Company: Fbrica III &lt;/p&gt;
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 2.0
 */

import java.util.*;

public class Horario implements Comparable {

  public static final String DOMINGO = "Domingo";

  public static final String SEGUNDA = "Segunda";

  public static final String TERCA = "Tera";

  public static final String QUARTA = "Quarta";

  public static final String QUINTA = "Quinta";

  public static final String SEXTA = "Sexta";

  public static final String SABADO = "Sbado";

  /*
   * Cdigo do horario.
   */
  private int codigo;

  /*
   * Dia da semana do horario.
   */
  private String diaDaSemana;

  /*
   * Hora inicio de horario.
   */
  private String horaInicio;

  /*
   * Hora final de horario.
   */
  private String horaFim;

  private HashMap diasSemana = new HashMap();
  /*
   * Construtor vazio.
   */
  public Horario() {
    this.diasSemana.put(DOMINGO, new Integer(1));
    this.diasSemana.put(SEGUNDA, new Integer(2));
    this.diasSemana.put(TERCA, new Integer(3));
    this.diasSemana.put(QUARTA, new Integer(4));
    this.diasSemana.put(QUINTA, new Integer(5));
    this.diasSemana.put(SEXTA, new Integer(6));
    this.diasSemana.put(SABADO, new Integer(7));
  }

  /*
   * Construtor
   */
  public Horario(int codigo, String diaDaSemana, String horaInicio, String horaFim) {
    this();
    setCodigo(codigo);
    setDiaDaSemana(diaDaSemana);
    setHoraInicio(horaInicio);
    setHoraFim(horaFim);
  }

  /**
   * Pega cdigo do horario.
   * @return int
   */
  public int getCodigo() {
    return this.codigo;
  }

  /**
   * Define cdigo do horario.
   * @param codigo
   */
  public void setCodigo(int codigo) {
    this.codigo = codigo;
  }

  /**
   * Pega dia da semana do horario.
   * @return String
   */
  public String getDiaDaSemana() {
    return this.diaDaSemana;
  }

  /**
   * Define dia da semana do horario.
   * @param diaDaSemana
   */
  public void setDiaDaSemana(String diaDaSemana) {
    this.diaDaSemana = diaDaSemana;
  }

  /**
   * Pega hora de incio do horario.
   * @return String
   */
  public String getHoraInicio() {
    return this.horaInicio;
  }

  /**
   * Define hora final do horario.
   * @param horaFim
   */
  public void setHoraInicio(String horaInicio) {
    this.horaInicio = horaInicio;
  }

  /**
   * Define hora final do horario.
   * @param horaFim
   */
  public void setHoraFim(String horaFim) {
    this.horaFim = horaFim;
  }

  /**
   * Pega hora final do horario.
   * @return String
   */
  public String getHoraFim() {
    return this.horaFim;
  }

  /**
   * Compara dois objetos horario.
   * @param horario
   * @return boolean
   */
  public boolean equals(Object horarioObjeto) {
    boolean result = false;
    Horario horario = (Horario)horarioObjeto;
    if((this.horaInicio.equals(horario.getHoraInicio())) &&
       (this.horaFim.equals(horario.getHoraFim())) )

      result = true;

    return result;
  }

  public int hashCode() {
    return 1;
  }

  public int getValorDiaSemana(String diaDaSemana) {
    return ((Integer)this.diasSemana.get(diaDaSemana)).intValue();
  }

  /**
   * Compara dois objetos horario, leva em considerao o dia da semana e a hora de
   inicio.
   * @return Um nmero negativo se o this for menor, zero se os objetos forem iguais
   e um
   * nmero positivo se o objeto this for maior.
   */
  public int compareTo(Object objeto) {
    Horario horario = (Horario)objeto;

    int diferencaDias = ((Integer)diasSemana.get(this.diaDaSemana)).intValue()
                        - ((Integer)diasSemana.get(horario.getDiaDaSemana())).intValue();

    int diferencaHoras = Integer.parseInt(this.horaInicio.substring(0,
        this.horaInicio.indexOf(":")))
                        - Integer.parseInt(horario.getHoraInicio().substring(0,
                        horario.getHoraInicio().indexOf(":")));

   int diferencaMinutos =
        Integer.parseInt(this.horaInicio.substring(this.horaInicio.indexOf(":")+1))
                        -
                        Integer.parseInt(horario.getHoraInicio().substring(horario.getHoraInicio().indexOf(":")+1));

        if (diferencaDias == 0){
          if (diferencaHoras == 0)
            return diferencaMinutos;
          else
                  return diferencaHoras;
        } else {
          return diferencaDias;
        }
  }

  /**
   * Concatena atributos do objeto horario em uma string.
   * @return String
   */
  public String toString() {
    StringBuffer str = new StringBuffer();

    str.append("sialocacao.alocacao.Horario: (");
    str.append(this.codigo);
    str.append(", ");
    str.append(this.diaDaSemana);
    str.append(", ");
    str.append(this.horaInicio);
    str.append(", ");
    str.append(this.horaFim);

    str.append(")");

    return str.toString();
  }


}