package sialocacao.alocacao.database;

/**
 * <p>Title: AlocacaoDatabase.java </p>
 * <p>Description: Implementa a camada de persistncia de Alocacao.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.sql.*;
import java.util.Date;
import java.util.ArrayList;

import sialocacao.*;
import sialocacao.alocacao.*;
import sialocacao.alocacao.excecoes.*;

import sialocacao.professor.*;
import sialocacao.professor.excecoes.*;
import sialocacao.disciplina.*;
import sialocacao.sala.*;
import sialocacao.sala.excecoes.*;
import sialocacao.util.*;

public class AlocacaoDatabase implements IAlocacaoDatabase {

  private Database poolBD;

  /*
   * Construtor para carregar driver jdbc.
   */
  public AlocacaoDatabase() throws Exception {
    poolBD = Database.instance();
  }

  /*
   * Insere uma Alocacao no sistema.
   * @param codSala
   * @param codHorario
   * @param codDisciplina
   * @param codProfessor
   * @param data
   */
    public void alocar(int codSala, int codHorario, int codDisciplina,
                       int codProfessor, Data data) throws SIAlocacaoException {

        Connection con = null;
        try {
            con = poolBD.getConnection();

            PreparedStatement cmd =
                con.prepareStatement("SELECT * FROM alocacao " +
                                                 "WHERE data=? AND codigoHorario=? AND (  " +
                                                 "codigoProfessor=? OR codigoSala=? )" );

            cmd.setString(1, data.toString());
            cmd.setInt(2, codHorario);
            cmd.setInt(3, codProfessor);
            cmd.setInt(4, codSala);

            ResultSet result = cmd.executeQuery();

            if (result.next()) {
              if(result.getInt("codigoProfessor") == codProfessor) {
                throw new ProfessorAlocadoException("Professor j alocado no horario " +
                                              "selecionado");
              } else {
                throw new SalaAlocadaException("Sala j alocada no horario " +
                                              "selecionado");
              }
            }

            result.close();

            cmd = con.prepareStatement("INSERT INTO alocacao (codigoHorario," +
                                                   "codigoSala, codigoDisciplina, codigoProfessor, "
                                                   + "data) VALUES (?, ?, ?, ?, ?)");

            cmd.setInt(1, codHorario);
            cmd.setInt(2, codSala);
            cmd.setInt(3, codDisciplina);
            cmd.setInt(4, codProfessor);
            cmd.setString(5, data.toString());

            cmd.executeUpdate();

            cmd.close();
        }catch(SIAlocacaoException e){
           throw e;
         }
         catch (Exception e) {
            throw new SIAlocacaoException(e.getMessage(), e);
        } finally {
            try {
              poolBD.returnConnection(con);
            } catch (Exception e) {
              throw new SIAlocacaoException(e.getMessage(), e);
            }
        }
    }

  /*
   * Retorna um objeto disciplina contendo as informaoes da disciplina alocada
   * @param sala
   * @param horario
   * @param data
   * @return Disciplina
   */
  public Disciplina getDisciplinaAlocada(Sala sala, Horario horario, Data data)
                                          throws SIAlocacaoException {

    Connection con = null;

    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("SELECT disciplina.codigo, disciplina.descricao,"
                                          + " disciplina.cargaHoraria, disciplina.ementa " +
                                           "FROM alocacao, disciplina " +
                                           "WHERE data=? AND codigoSala=? AND codigoHorario=? " +
                                           "AND codigoDisciplina=disciplina.codigo");

      cmd.setString(1, data.toString());
      cmd.setInt(2, sala.getCodigo());
      cmd.setInt(3, horario.getCodigo());

      ResultSet result = cmd.executeQuery();

      Disciplina disciplina = null;

      if (result.next()) {
        disciplina = new Disciplina(result.getInt("codigo"),
                        result.getString("descricao"),
                        result.getInt("cargaHoraria"),
                        result.getString("ementa") );
      }

      result.close();
      cmd.close();
      return disciplina;
    } catch (Exception e) {
      e.printStackTrace();
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Retorna o objeto alocacao para sala e horario especificados.
   * @param sala
   * @param horario
   * @param data
   * @return Alocacao
   */
  public Alocacao getAlocacao(Sala sala, Horario horario, Data data) throws SIAlocacaoException {

    Connection con = null;

    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("SELECT codigo, codigoProfessor, " +
                                           "codigoDisciplina FROM alocacao " +
                                           "WHERE data=? AND codigoHorario=? AND " +
                                           "codigoSala=?");

      cmd.setString(1, data.toString());
      cmd.setInt(2, horario.getCodigo());
      cmd.setInt(3, sala.getCodigo());


      ResultSet result = cmd.executeQuery();

      Alocacao alocacao;

      if (result.next()) {
        alocacao = new Alocacao(result.getInt("codigo"),
                                sala.getCodigo(), horario.getCodigo(),
                                result.getInt("codigoDisciplina"),
                                result.getInt("codigoProfessor"),
                                data);
      } else {
        throw new AlocacaoInexistenteException("Alocao especificada no existe!");
      }

      result.close();
      cmd.close();

      return alocacao;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }


  /*
   * Altera Alocacao no sistema.
   * @param alocacao
   */
  public void alterarAlocacao(Alocacao alocacao) throws SIAlocacaoException {

    Connection con = null;

    try {

      con = poolBD.getConnection();

      // Verifica se o novo professor/sala a serem alocados esto disponveis
      PreparedStatement cmd =
          con.prepareStatement("SELECT * FROM alocacao " +
                                           "WHERE data=? AND codigoHorario=? "+
                                           "AND (codigoProfessor=? OR codigoSala=?)");

      cmd.setString(1, (alocacao.getData()).toString());
      cmd.setInt(2, alocacao.getCodHorario());
      cmd.setInt(3, alocacao.getCodProfessor());
      cmd.setInt(4, alocacao.getCodSala());

      ResultSet result = cmd.executeQuery();

      if (result.next()) {
        boolean professorAlocado = result.getInt("codigoProfessor") == alocacao.getCodProfessor();
        if( professorAlocado &&
           (result.getInt("codigoSala") != alocacao.getCodSala())) {
          throw new SIAlocacaoException("Professor j alocado no horario " +
                                        "selecionado");
        } /*else if(!professorAlocado ) {
          throw new SIAlocacaoException("Sala j alocada no horario " +
                                        "selecionado");
        }*/
      }

      result.close();

      cmd = con.prepareStatement( "UPDATE alocacao SET " +
                                              "codigoProfessor=?, codigoSala=?, " +
                                              "codigoHorario=?, codigoDisciplina=?, " +
                                              "data=? WHERE codigo=?");

      cmd.setInt(1, alocacao.getCodProfessor());
      cmd.setInt(2, alocacao.getCodSala());
      cmd.setInt(3, alocacao.getCodHorario());
      cmd.setInt(4, alocacao.getCodDisciplina());
      cmd.setString(5, (alocacao.getData()).toString());
      cmd.setInt(6, alocacao.getCodigo());

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new AlocacaoInexistenteException(alocacao.getCodigo());
      }

      cmd.close();

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

 /*
   * Exclui Alocacao do sistema.
   * @param codigo
   */
  public void excluirAlocacao(int codigo) throws SIAlocacaoException {

    Connection con = null;

    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("DELETE FROM alocacao WHERE " +
                                           "codigo=?");

      cmd.setInt(1, codigo );

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new AlocacaoInexistenteException(codigo);
      }

      cmd.close();

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Lista alocacoes do sistema.
   * @return ArrayList
   */
  public ArrayList listarAlocacoes() throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("SELECT * FROM alocacao");

      ResultSet result = cmd.executeQuery();

      ArrayList alocacoes = new ArrayList();

      while (result.next()) {
        alocacoes.add(new Alocacao (result.getInt("codigo"),
                                    result.getInt("codigoSala"),
                                    result.getInt("codigoHorario"),
                                    result.getInt("codigoDisciplina"),
                                    result.getInt("codigoProfessor"),
                                    new Data(result.getString("data"))));
      }

      result.close();
      cmd.close();
      return alocacoes;

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  } // listarAlocacoes

  /*
   * Lista horarios do sistema.
   * @return ArrayList
   */
  public ArrayList listarHorarios() throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("SELECT * FROM horario ORDER BY "+
            "codigo");

      ResultSet result = cmd.executeQuery();

      ArrayList horarios =
          new ArrayList();

      while (result.next()) {
        horarios.add(
            new Horario(result.getInt("codigo"),
                        result.getString("diaDaSemana"),
                        result.getString("horaInicio"),
                        result.getString("horaFim")));
      }
      result.close();
      cmd.close();
      return horarios;

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  } //listarHorarios

  /*
   * Busca horario no sistema pela hora de inicio, hora final e dia da semana.
   * @param horaInicio
   * @param horaFim
   * @param diaSemana
   * @return Professor
   */
  public Horario buscarHorario(String diaDaSemana, String horaInicio, String horaFim)
                               throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();

      PreparedStatement cmd =
          con.prepareStatement("SELECT codigo FROM horario WHERE "+
            "diaDaSemana=? AND horaInicio=? AND horaFim=?");

      cmd.setString(1, diaDaSemana);
      cmd.setString(2, horaInicio);
      cmd.setString(3, horaFim);

      ResultSet result = cmd.executeQuery();

      Horario horario;

      if (result.next()) {
        horario =
            new Horario(result.getInt("codigo"),
                        diaDaSemana,
                        horaInicio,
                        horaFim);
      } else {
        throw new HorarioInexistenteException("Horrio especificado no existe!");
      }

      result.close();
      cmd.close();
      return horario;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Busca horario no sistema pelo codigo do horario.
   * @param codigoHorario
   * @return Professor
   */
  public Horario buscarHorario(int codigoHorario) throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();

      PreparedStatement cmd =
          con.prepareStatement("SELECT diaDaSemana, horaInicio, horaFim FROM "+
           "horario WHERE codigo=?");

      cmd.setInt(1, codigoHorario);

      ResultSet result = cmd.executeQuery();

      Horario horario;

      if (result.next()) {
        horario =
            new Horario(codigoHorario,
                        result.getString("diaDaSemana"),
                        result.getString("horaInicio"),
                        result.getString("horaFim"));
      } else {
        throw new HorarioInexistenteException("Horrio especificado no existe!");
      }

      result.close();
      cmd.close();
      return horario;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        poolBD.returnConnection(con);
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }
}
