package sialocacao.alocacao.database;

import java.util.*;
import sialocacao.*;
import sialocacao.alocacao.*;
import sialocacao.alocacao.excecoes.*;

import sialocacao.professor.*;
import sialocacao.disciplina.*;
import sialocacao.sala.*;
import sialocacao.util.*;


/**
 * <p>Title: IAlocacaoDatabase.java </p>
 * <p>Description: Interface da classe AlocacaoDatabase.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III</p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

public interface IAlocacaoDatabase {

  /*
   * Insere uma Alocacao no sistema.
   * @param codSala
   * @param codHorario
   * @param codDisciplina
   * @param codProfessor
   * @param data
   */
  public void alocar(int codSala, int codHorario, int codDisciplina,
                     int codProfessor, Data data) throws SIAlocacaoException;

  /*
   * Retorna um objeto disciplina contendo as informaoes da disciplina alocada
   * @param sala
   * @param horaInicio
   * @param horaFim
   * @param horario
   * @return Disciplina
   */
  public Disciplina getDisciplinaAlocada(Sala sala, Horario horario,
                                   Data data) throws SIAlocacaoException;

  /*
   * Retorna o objeto alocacao para sala e horario especificados.
   * @param sala
   * @param horaInicio
   * @param horaFim
   * @param data
   * @return Alocacao
   */
  public Alocacao getAlocacao(Sala sala, Horario horario, Data data)
                              throws SIAlocacaoException;

  /*
   * Altera Alocacao no sistema.
   * @param alocacao
   */
  public void alterarAlocacao(Alocacao alocacao) throws SIAlocacaoException;

  /*
   * Exclui Alocacao do sistema.
   * @param alocacao
   */
  public void excluirAlocacao(int codigo) throws SIAlocacaoException;

  /*
   * Lista alocacoes do sistema.
   * @return ArrayList
   */
  public ArrayList listarAlocacoes() throws SIAlocacaoException;

  /*
   * Lista horarios do sistema.
   * @return ArrayList
   */
  public ArrayList listarHorarios() throws SIAlocacaoException;

  /*
   * Retorna um objeto horario correspondente aos parametros passados
   * @param diadaSemana
   * @param horaInicio
   * @param horaFim
   * @return Horario
   */
  public Horario buscarHorario(String diaDaSemana, String horaInicio, String horaFim)
                               throws SIAlocacaoException;

  /*
   * Busca horario no sistema pelo codigo do horario.
   * @param codigoHorario
   * @return Professor
   */
  public Horario buscarHorario(int codigoHorario) throws SIAlocacaoException;
}