package sialocacao.alocacao.excecoes;

/**
 * <p>Title: HorarioInexistenteException.java </p>
 * <p>Description: Responsvel por tratar exceo de Horario inexistente. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import sialocacao.*;
import sialocacao.alocacao.*;

public class HorarioInexistenteException extends SIAlocacaoException {

  /*
   * Construtor responsvel por definir o cdigo do horrio inexistente.
   */
  public HorarioInexistenteException(int codHorario) {
    setDescription(getDescription(codHorario));
  }

  /*
   * Construtor responsvel por definir a descricao do horrio inexistente.
   */
  public HorarioInexistenteException(String descricao) {
    setDescription(getDescription(descricao));
  }

  /*
   * Construtor responsvel por definir o horario inexistente.
   */
  public HorarioInexistenteException(Horario horario)
      throws IllegalArgumentException {
    if (horario == null) {
      throw new IllegalArgumentException("horario == null");
    }
    setDescription(getDescription(horario.getCodigo()));
  }

  /*
   * Define descrio da exceo, passando o cdigo do horario inexistente.
   * @param codHorario
   */
  private String getDescription(int codHorario) {
    StringBuffer description = new StringBuffer( "Nao existe horario com o codigo ");

    description.append(codHorario);

    return description.toString();
  }

  /*
   * Define descrio da exceo, passando a descricao do horario inexistente.
   * @param descricao
   */
  private String getDescription(String descricao) {
    StringBuffer description = new StringBuffer( "Nao existe horario com o codigo ");

    description.append(descricao);

    return description.toString();
  }
}