package sialocacao.alocacao.negocio;

/**
 * <p>Title: CadastroAlocacoes.java </p>
 * <p>Description: Implementa a camanda de negcio de alocacao. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.util.*;

import sialocacao.*;
import sialocacao.util.*;
import sialocacao.alocacao.*;
import sialocacao.alocacao.database.*;
import sialocacao.alocacao.excecoes.*;

import sialocacao.professor.*;
import sialocacao.disciplina.*;
import sialocacao.sala.*;

import sialocacao.professor.negocio.*;
import sialocacao.disciplina.negocio.*;
import sialocacao.sala.negocio.*;

import sialocacao.professor.excecoes.*;
import sialocacao.disciplina.excecoes.*;
import sialocacao.sala.excecoes.*;

public class CadastroAlocacoes implements ICadastroAlocacoes {

  /*
   * Instanica da Interface da camada de negcios de disciplinas.
   */
  private ICadastroDisciplinas cadastroDisciplinas;

  /*
   * Instanica da Interface da camada de negcios de salas.
   */
  private ICadastroSalas cadastroSalas;

  /*
   * Instncia do repositrio de alocacoes.
   */
  private IAlocacaoDatabase alocacaoDatabase;

  /*
   * Construtor definindo o repositrio de alocacoes.
   * @param
   */
  public CadastroAlocacoes() throws Exception{
    setAlocacaoDatabase(new AlocacaoDatabase());
    setCadastroDisciplinas( new CadastroDisciplinas() );
    setCadastroSalas( new CadastroSalas() );
  }

  /*
   * Construtor.
   * @param alocacaoDatabase
   * @param cadastroDisciplinas
   * @param cadastroSalas
   */
  public CadastroAlocacoes(IAlocacaoDatabase alocacaoDatabase,
                         ICadastroDisciplinas cadastroDisciplinas,
                         ICadastroSalas cadastroSalas) {
    setAlocacaoDatabase(alocacaoDatabase);
    setCadastroDisciplinas(cadastroDisciplinas);
    setCadastroSalas(cadastroSalas);
   }

   /*
    * Define repositrio de alocacoes.
    * @param disciplinaDatabase
    */
   public void setAlocacaoDatabase(IAlocacaoDatabase alocacaoDatabase)
       throws IllegalArgumentException {

     if (alocacaoDatabase == null) {
       throw new IllegalAccessError("alocacaoDatabase == null");
     }

     this.alocacaoDatabase = alocacaoDatabase;
   }

   /*
    * Define instanica da interface da camada de negcios de disciplina.
    * @param disciplinaDatabase
    */
   public void setCadastroDisciplinas ( ICadastroDisciplinas cadastroDisciplinas )
       throws IllegalArgumentException {

     if (cadastroDisciplinas == null) {
       throw new IllegalAccessError("cadastroDisciplinas == null");
     }

     this.cadastroDisciplinas = cadastroDisciplinas;
   }

   /*
    * Define instanica da interface da camada de negcios de sala.
    * @param cadastroSalas
    */
   public void setCadastroSalas ( ICadastroSalas cadastroSalas )
       throws IllegalArgumentException {

     if (cadastroSalas == null) {
       throw new IllegalAccessError("cadastroSalas == null");
     }

     this.cadastroSalas = cadastroSalas;
   }


   /*
    * Insere uma Alocacao no sistema.
    * @param horario
    * @param sala
    * @param disciplina
    * @param professor
    */


   public void alocar(Sala sala, Disciplina disciplina, Horario horario,
                      Professor professor, Data data) throws SIAlocacaoException {

     try{
       cadastroDisciplinas.buscarDisciplina(disciplina.getCodigo());
     } catch ( DisciplinaInexistenteException e ) {
       throw new SIAlocacaoException("Disciplina no cadastrada");
     }

     try{
       cadastroSalas.buscarSala(sala.getCodigo());
     } catch ( SalaInexistenteException e ) {
       throw new SIAlocacaoException("Sala no cadastrada");
     }


     alocacaoDatabase.alocar(sala.getCodigo(), horario.getCodigo(), disciplina.getCodigo(),
                  professor.getCodigo(), data);

   }

   /*
    * Retorna um objeto disciplina contendo as informaoes da disciplina alocada
    * no horario e sala especificados.
    * @param horario
    * @return sala
    */
   public Disciplina getDisciplinaAlocada(Sala sala, Horario horario, Data data) throws SIAlocacaoException{
     return alocacaoDatabase.getDisciplinaAlocada(sala, horario, data);
   }

   /*
    * Exclui Alocacao do sistema.
    * @param alocacao
    */
   public void excluirAlocacao(int codigo) throws SIAlocacaoException{
     alocacaoDatabase.excluirAlocacao(codigo);
   }

   /*
    * Altera Alocacao no sistema.
    * @param alocacao
    */
   public void alterarAlocacao(Alocacao alocacao) throws SIAlocacaoException {
     try{
       cadastroDisciplinas.buscarDisciplina(alocacao.getCodDisciplina());
     } catch ( DisciplinaInexistenteException e ) {
       throw new SIAlocacaoException("Disciplina no cadastrada");
     }

     try{
       cadastroSalas.buscarSala(alocacao.getCodSala());
     } catch ( SalaInexistenteException e ) {
       throw new SIAlocacaoException("Sala no cadastrada");
     }

     alocacaoDatabase.alterarAlocacao(alocacao);
   }

   /*
    * Lista alocacoes do sistema.
    * @return ArrayList
    */
   public ArrayList listarAlocacoes() throws SIAlocacaoException {
     return alocacaoDatabase.listarAlocacoes();
   }

   /*
    * Retorna o objeto alocacao para horario e sala especificados.
    * @param horario
    * @return sala
    */
   /*
   public Alocacao getAlocacao(Sala sala, String horaInicio, String horaFim,
                              String diaSemana, Date data) throws SIAlocacaoException,
       HorarioNaoAlocadoException
   {
     return alocacaoDatabase.getAlocacao(sala, horaInicio, horaFim, diaSemana, data);
   }
   */
   /*
    * Retorna o objeto alocacao para horario e sala especificados.
    * @param horario
    * @return sala
    */
   public Alocacao getAlocacao(Sala sala, Horario horario, Data data) throws SIAlocacaoException {
      return alocacaoDatabase.getAlocacao(sala, horario, data);
   }

  /*
   * Lista horarios do sistema.
   * @return ArrayList
   */
  public ArrayList listarHorarios() throws SIAlocacaoException{
    return alocacaoDatabase.listarHorarios();
  }

  /*
   * Retorna um objeto horario correspondente aos parametros passados
   * @param diadaSemana
   * @param horaInicio
   * @param horaFim
   * @return Horario
   */
  public Horario buscarHorario(String diaDaSemana, String horaInicio, String horaFim)
                               throws SIAlocacaoException {
    return alocacaoDatabase.buscarHorario(diaDaSemana, horaInicio, horaFim);
  }

  /*
   * Busca horario no sistema pelo codigo do horario.
   * @param codigoHorario
   * @return Professor
   */
  public Horario buscarHorario(int codigoHorario) throws SIAlocacaoException{
    return alocacaoDatabase.buscarHorario(codigoHorario);
  }

/*
   /*
    * Insere uma Alocacao no sistema.
    * @param horario
    * @param sala
    * @param disciplina
    * @param professor
    *
   public void alocar(Sala sala, Disciplina disciplina, String horaInicio,
                  String horaFim, String diaSemana, Date periodoInicio,
                  Date periodoFinal) throws
       SIAlocacaoException {

     try{
       cadastroDisciplinas.buscarDisciplina(disciplina.getCodigo());
     } catch ( DisciplinaInexistenteException e ) {
       throw new SIAlocacaoException("Disciplina no cadastrada");
     }

     try{
       cadastroSalas.buscarSala(sala.getCodigo());
     } catch ( SalaInexistenteException e ) {
       throw new SIAlocacaoException("Sala no cadastrada");
     }


//     alocacaoDatabase.alocar(sala, disciplina, horaInicio,
//                  horaFim, diaSemana, periodoInicio, periodoFinal);

   }

   /*
    * Retorna um objeto disciplina contendo as informaoes da disciplina alocada
    * no horario e sala especificados.
    * @param horario
    * @return sala
    *
   public Disciplina getDisciplinaAlocada(Sala sala, String horaInicio, String horaFim,
                                         String diaSemana) throws SIAlocacaoException{
     return null;//alocacaoDatabase.getDisciplinaAlocada(sala, horaInicio, horaFim, diaSemana);
   }

   /*
    * Exclui Alocacao do sistema.
    * @param alocacao
    *
   public void excluirAlocacao(int codigo) throws SIAlocacaoException{
     alocacaoDatabase.excluirAlocacao(codigo);
   }

   /*
    * Altera Alocacao no sistema.
    * @param alocacao
    *
   public void alterarAlocacao(Alocacao alocacao)
       throws SIAlocacaoException, AlocacaoInexistenteException
   {
     try{
       cadastroDisciplinas.buscarDisciplina(alocacao.getCodDisciplina());
     } catch ( DisciplinaInexistenteException e ) {
       throw new SIAlocacaoException("Disciplina no cadastrada");
     }

     try{
       cadastroSalas.buscarSala(alocacao.getCodSala());
     } catch ( SalaInexistenteException e ) {
       throw new SIAlocacaoException("Sala no cadastrada");
     }

     alocacaoDatabase.alterarAlocacao(alocacao);
   }

   /*
    * Lista alocacoes do sistema.
    * @return ArrayList
    *
   public ArrayList listarAlocacoes() throws SIAlocacaoException {
     return alocacaoDatabase.listarAlocacoes();
   }

   /*
    * Retorna o objeto alocacao para horario e sala especificados.
    * @param horario
    * @return sala
    *
   public Alocacao getAlocacao(Sala sala, String horaInicio, String horaFim,
                              String diaSemana, Date data) throws SIAlocacaoException,
       HorarioNaoAlocadoException
   {
     return null;//alocacaoDatabase.getAlocacao(sala, horaInicio, horaFim, diaSemana, data);
   }

  /*
   * Lista horarios do sistema.
   * @return ArrayList
   *
  public ArrayList listarHorarios() throws SIAlocacaoException{
    return alocacaoDatabase.listarHorarios();
  }
*/
}