package sialocacao.alocacao.negocio;

/**
 * <p>Title: ICadastroAlocacoes.java </p>
 * <p>Description: Interface da classe CadastroAlocacoes. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.util.*;

import sialocacao.*;
import sialocacao.alocacao.*;
import sialocacao.alocacao.database.*;
import sialocacao.alocacao.excecoes.*;
import sialocacao.professor.*;

import sialocacao.disciplina.*;
import sialocacao.sala.*;

import sialocacao.disciplina.negocio.*;
import sialocacao.sala.negocio.*;
import sialocacao.util.*;

public interface ICadastroAlocacoes {

  /*
   * Define repositrio de alocacao.
   * @param disciplinaDatabase
   */
  public void setAlocacaoDatabase(IAlocacaoDatabase alocacaoDatabase)
      throws IllegalArgumentException;

  /*
   * Define instanica da interface da camada de negcios de disciplina.
   * @param disciplinaDatabase
   */
  public void setCadastroDisciplinas ( ICadastroDisciplinas cadastroDisciplinas )
      throws IllegalArgumentException ;

  /*
   * Define instanica da interface da camada de negcios de sala.
   * @param cadastroSalas
   */
  public void setCadastroSalas ( ICadastroSalas cadastroSalas )
      throws IllegalArgumentException ;


  /*
   * Insere uma Alocacao no sistema.
   * @param horario
   * @param sala
   * @param disciplina
   * @param professor
   */
  public void alocar(Sala sala, Disciplina disciplina, Horario horario,
                      Professor professor, Data data) throws
      SIAlocacaoException;

  /*
   * Retorna um objeto disciplina contendo as informaoes da disciplina alocada
   * no horario e sala especificados.
   * @param horario
   * @return sala
   */
   public Disciplina getDisciplinaAlocada(Sala sala, Horario horario, Data data)
       throws SIAlocacaoException;

  /*
   * Exclui Alocacao do sistema.
   * @param alocacao
   */
  public void excluirAlocacao(int codigo) throws SIAlocacaoException;

  /*
   * Altera Alocacao no sistema.
   * @param alocacao
   */
  public void alterarAlocacao(Alocacao alocacao) throws SIAlocacaoException;

  /*
   * Lista alocacoes do sistema.
   * @return ArrayList
   */
  public ArrayList listarAlocacoes() throws SIAlocacaoException;


  /*
   * Retorna o objeto alocacao para horario e sala especificados.
   * @param horario
   * @return sala
   *
  public Alocacao getAlocacao(Sala sala, String horaInicio, String horaFim,
                              String diaSemana, Date data) throws SIAlocacaoException,
      HorarioNaoAlocadoException;
*/
  public Alocacao getAlocacao(Sala sala, Horario horario, Data data) throws SIAlocacaoException;

  /*
   * Lista horarios do sistema.
   * @return ArrayList
   */
  public ArrayList listarHorarios() throws SIAlocacaoException;

  /*
   * Retorna um objeto horario correspondente aos parametros passados
   * @param diadaSemana
   * @param horaInicio
   * @param horaFim
   * @return Horario
   */
  public Horario buscarHorario(String diaDaSemana, String horaInicio, String horaFim)
                               throws SIAlocacaoException;

  /* Retorna um objeto horario ao codigoHorario
   * @param diadaSemana
   * @param horaInicio
   * @param horaFim
   * @return Horario
   */
  public Horario buscarHorario(int codigoHorario) throws SIAlocacaoException;
}