package sialocacao.disciplina.negocio;

/**
 * <p>Title: CadastroDisciplinas.java </p>
 * <p>Description: Implementa a camanda de negcio de disciplinas. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.util.*;

import sialocacao.*;
import sialocacao.disciplina.*;
import sialocacao.disciplina.database.*;
import sialocacao.disciplina.excecoes.*;

public class CadastroDisciplinas implements ICadastroDisciplinas {

  /*
   * Instncia do repositrio de disciplinas.
   */
   private IDisciplinaDatabase disciplinaDatabase;

  /*
   * Construtor definindo o repositrio de disciplinas.
   * @param disciplinaDatabase
   */
   public CadastroDisciplinas() throws Exception {
     setDisciplinaDatabase(new DisciplinaDatabase());
   }

  /*
   * Construtor definindo o repositrio de disciplinas.
   * @param disciplinaDatabase
   */
   public CadastroDisciplinas(IDisciplinaDatabase disciplinaDatabase) {
     setDisciplinaDatabase(disciplinaDatabase);
   }

  /*
   * Define repositrio de disciplinas.
   * @param disciplinaDatabase
   */
   public void setDisciplinaDatabase(IDisciplinaDatabase disciplinaDatabase)
      throws IllegalArgumentException {

    if (disciplinaDatabase == null) {
      throw new IllegalAccessError("disciplinaDatabase == null");
    }

    this.disciplinaDatabase = disciplinaDatabase;
  }

  /*
   * Insere disciplina no sistema.
   * @param descricao
   * @param cargaHoraria
   * @param ementa
   */
  public void inserirDisciplina(String descricao, int cargaHoraria, String ementa, Map preferencias)
      throws SIAlocacaoException {

    disciplinaDatabase.inserirDisciplina(descricao, cargaHoraria, ementa, preferencias);
  }

  /*
   * Altera disciplina no sistema.
   * @param disciplina
   */
  public void alterarDisciplina(Disciplina disciplina, Map prioridades)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    disciplinaDatabase.alterarDisciplina(disciplina, prioridades);
  }

  /*
   * Exclui disciplina do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    disciplinaDatabase.excluirDisciplina(codigo);
  }

  /*
   * Busca disciplina no sistema pelo cdigo.
   * @param codigo
   * @return Disciplina
   */
  public Disciplina buscarDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    return disciplinaDatabase.buscarDisciplina(codigo);
  }

  /*
   * Busca disciplina no sistema pelo cdigo.
   * @param descricao
   * @return Disciplina
   */
  public Disciplina buscarDisciplina(String descricao)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    return disciplinaDatabase.buscarDisciplina(descricao);
  }

  /*
   * Lista disciplinas do sistema.
   * @return ArrayList
   */
  public ArrayList listarDisciplinas() throws
      SIAlocacaoException, DisciplinaInexistenteException {

    return disciplinaDatabase.listarDisciplinas();
  }

  public Map buscarPrioridades(Disciplina disciplina, List categorias) throws
      SIAlocacaoException {

    return this.disciplinaDatabase.buscarPrioridades(disciplina, categorias);
  }

  /*
   * Associa disciplina a professor no sistema.
   * @param disciplina
   * @param codProfessor
   */
  public void associaProfessorDisciplina(Disciplina disciplina, int codProfessor)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    disciplinaDatabase.associaProfessorDisciplina(disciplina, codProfessor);
  }

  /*
   * Remove associacao de disciplina a professor no sistema.
   * @param codProfessor
   */
  public void removeProfessorDisciplina(int codProfessor) throws
       SIAlocacaoException, DisciplinaInexistenteException {

    disciplinaDatabase.removeProfessorDisciplina(codProfessor);
  }

  /*
   * Busca codigo do profesor associado a disciplina no sistema.
   * @param codigo
   * @return int
   */
  public int buscarProfessorDeDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    return disciplinaDatabase.buscarProfessorDeDisciplina(codigo);
  }

  /*
   * Verifica se todas as disciplinas estao associadas a um professor
   * @return true
   */
  public boolean todasDisciplinasAssociadas()
      throws SIAlocacaoException{

    return disciplinaDatabase.todasDisciplinasAssociadas();
  }
}