package sialocacao.disciplina.negocio;

/**
 * <p>Title: ICadastroDisciplinas.java </p>
 * <p>Description: Interface da classe CadastroDisciplinas. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.util.*;

import sialocacao.*;
import sialocacao.disciplina.*;
import sialocacao.disciplina.database.*;
import sialocacao.disciplina.excecoes.*;

public interface ICadastroDisciplinas {

  /*
   * Define repositrio de disciplinas.
   * @param disciplinaDatabase
   */
  public void setDisciplinaDatabase(IDisciplinaDatabase disciplinaDatabase)
      throws IllegalArgumentException;

  /*
   * Insere disciplina no sistema.
   * @param descricao
   * @param cargaHoraria
   * @param ementa
   */
  public void inserirDisciplina(String descricao, int cargaHoraria, String ementa, Map preferencias)
      throws SIAlocacaoException;

  /*
   * Altera disciplina no sistema.
   * @param disciplina
   */
  public void alterarDisciplina(Disciplina disciplina, Map prioridades)
      throws SIAlocacaoException, DisciplinaInexistenteException;

  /*
   * Exclui disciplina do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException;

  /*
   * Busca disciplina no sistema pelo cdigo.
   * @param codigo
   * @return Disciplina
   */
  public Disciplina buscarDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException;

  /*
   * Busca disciplina no sistema pelo cdigo.
   * @param descricao
   * @return Disciplina
   */
  public Disciplina buscarDisciplina(String descricao)
      throws SIAlocacaoException, DisciplinaInexistenteException;

  /*
   * Lista disciplinas do sistema.
   * @return ArrayList
   */
  public ArrayList listarDisciplinas() throws
      SIAlocacaoException, DisciplinaInexistenteException;

  public Map buscarPrioridades(Disciplina disciplina, List categorias) throws
      SIAlocacaoException;

  /*
   * Associa disciplina a professor no sistema.
   * @param disciplina
   * @param codProfessor
   */
  public void associaProfessorDisciplina(Disciplina disciplina, int codProfessor)
      throws SIAlocacaoException, DisciplinaInexistenteException;

  /*
   * Remove associacao de disciplina a professor no sistema.
   * @param codProfessor
   */
  public void removeProfessorDisciplina(int codProfessor) throws
       SIAlocacaoException, DisciplinaInexistenteException;

  /*
   * Busca codigo do profesor associado a disciplina no sistema.
   * @param codigo
   * @return int
   */
  public int buscarProfessorDeDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException;

  /*
   * Verifica se todas as disciplinas estao associadas a um professor
   * @return true
   */
  public boolean todasDisciplinasAssociadas()
      throws SIAlocacaoException;
}