package sialocacao.professor.negocio;

/**
 * <p>Title: CadastroProfessores.java </p>
 * <p>Description: Implementa a camanda de negcio de professores. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 2.0
 */

import java.util.*;

import sialocacao.*;
import sialocacao.professor.*;
import sialocacao.professor.database.*;
import sialocacao.professor.excecoes.*;

public class CadastroProfessores implements ICadastroProfessores {

  /*
   * Instncia do repositrio de professores.
   */
  private IProfessorDatabase professorDatabase;

  /*
   * Construtor default definindo o repositrio de professores.
   *
   */
  public CadastroProfessores() throws Exception{
    setProfessorDatabase(new ProfessorDatabase());
  }

  /*
   * Construtor definindo o repositrio de professores.
   * @param professorDatabase
   */
  public CadastroProfessores(IProfessorDatabase professorDatabase) {
    setProfessorDatabase(professorDatabase);
  }

  /*
   * Define repositrio de professores.
   * @param professorDatabase
   */
  public void setProfessorDatabase(IProfessorDatabase professorDatabase) throws
      IllegalArgumentException {

    if (professorDatabase == null) {
      throw new IllegalAccessError("professorDatabase == null");
    }

    this.professorDatabase = professorDatabase;
  }

  /*
   * Insere professor no sistema.
   * @param nome
   * @param cpf
   */
  public void inserirProfessor(String nome, String cpf) throws
      SIAlocacaoException, ProfessorJaCadastradoException {

    professorDatabase.inserirProfessor(nome, cpf);
  }

  /*
   * Altera professor no sistema.
   * @param professor
   */
  public void alterarProfessor(Professor professor) throws SIAlocacaoException,
      ProfessorInexistenteException {

    professorDatabase.alterarProfessor(professor);
  }

  /*
   * Exclui professor do sistema pelo codigo.
   * @param codigo
   */
  public void excluirProfessor(int codigo) throws SIAlocacaoException,
      ProfessorInexistenteException, ProfessorAlocadoException {

    professorDatabase.excluirProfessor(codigo);
  }

   /*
   * Busca professor no sistema pelo codigo.
   * @param codigo
   * @return Professor
   */
  public Professor buscarProfessor(int codigo) throws SIAlocacaoException,
      ProfessorInexistenteException {

    return professorDatabase.buscarProfessor(codigo);
  }

  /*
   * Busca professor no sistema pelo nome.
   * @param nome
   * @return Professor
   */
  public Professor buscarProfessor(String nome) throws SIAlocacaoException,
      ProfessorInexistenteException {

    return professorDatabase.buscarProfessor(nome);
  }

  /*
   * Lista professores do sistema.
   * @return ArrayList
   */
  public ArrayList listarProfessores() throws SIAlocacaoException {

    return professorDatabase.listarProfessores();
  }

  /*
   * Insere os horarios  preferenciais do professor no sistema.
   * @param codigoProfessor
   * @param codigoHorario
   * @param preferencia
   */
  public void inserirHorariosProfessor(int codigoProfessor, int codigoHorario,
      int prioridade) throws SIAlocacaoException {

    professorDatabase.inserirHorariosProfessor(codigoProfessor, codigoHorario,
                                               prioridade);
  }

  /*
   * Altera os horarios  preferenciais do professor no sistema.
   * @param codigoProfessor
   * @param codigoHorario
   * @param preferencia
   */
  public void alterarHorariosProfessor(int codigoProfessor, int codigoHorario,
      int prioridade) throws SIAlocacaoException {

    professorDatabase.alterarHorariosProfessor(codigoProfessor, codigoHorario,
                                               prioridade);
  }

  public Map buscarPrioridades(Professor professor) throws SIAlocacaoException{
    return professorDatabase.buscarPrioridades(professor);
  }

  public ArrayList listarPreferencias(int codProfessor)
    throws SIAlocacaoException{

    return professorDatabase.listarPreferencias(codProfessor);
  }
}