package sialocacao.professor.negocio;

/**
 * <p>Title: ICadastroProfessores.java </p>
 * <p>Description: Interface da classe CadastroProfessores. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 2.0
 */

import java.util.*;

import sialocacao.*;
import sialocacao.professor.*;
import sialocacao.professor.database.*;
import sialocacao.professor.excecoes.*;

public interface ICadastroProfessores {

  /*
   * Define repositrio de professores.
   * @param professorDatabase
   */
  public void setProfessorDatabase(IProfessorDatabase professorDatabase) throws
      IllegalArgumentException;

  /*
   * Insere professor no sistema.
   * @param nome
   * @param area
   */
  public void inserirProfessor(String nome, String cpf) throws
      SIAlocacaoException, ProfessorJaCadastradoException;

  /*
   * Altera professor no sistema.
   * @param professor
   */
  public void alterarProfessor(Professor professor) throws SIAlocacaoException,
      ProfessorInexistenteException;

  /*
   * Exclui professor do sistema pelo codigo.
   * @param codigo
   */
  public void excluirProfessor(int codigo) throws SIAlocacaoException,
      ProfessorInexistenteException;

  /*
   * Busca professor no sistema pelo codigo.
   * @param codigo
   * @return Professor
   */
  public Professor buscarProfessor(int codigo) throws SIAlocacaoException,
      ProfessorInexistenteException;

  /*
   * Busca professor no sistema pelo codigo.
   * @param nome
   * @return Professor
   */
  public Professor buscarProfessor(String nome) throws SIAlocacaoException,
      ProfessorInexistenteException;

  /*
   * Lista professores do sistema.
   * @return ArrayList
   */
  public ArrayList listarProfessores() throws SIAlocacaoException;

  /*
   * Insere os horarios  preferenciais do professor no sistema.
   * @param codigoProfessor
   * @param codigoHorario
   * @param preferencia
   */
  public void inserirHorariosProfessor(int codigoProfessor, int codigoHorario,
      int prioridade) throws SIAlocacaoException;

  /*
   * Altera os horarios  preferenciais do professor no sistema.
   * @param codigoProfessor
   * @param codigoHorario
   * @param preferencia
   */
  public void alterarHorariosProfessor(int codigoProfessor, int codigoHorario,
      int prioridade) throws SIAlocacaoException;

  public Map buscarPrioridades(Professor professor) throws SIAlocacaoException;

  public ArrayList listarPreferencias(int codProfessor)
    throws SIAlocacaoException;
}