package sialocacao.sala.database;

/**
 * <p>Title: SalaDatabase.java </p>
 * <p>Description: Implementa a camada de persistncia de sala. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.sql.*;
import java.util.*;

import sialocacao.*;
import sialocacao.sala.*;
import sialocacao.sala.excecoes.*;

public class CategoriaDatabase implements ICategoriaDatabase {

  private Database poolBD;

  /*
   * Construtor para carregar driver jdbc.
   */
  public CategoriaDatabase() throws Exception {
    poolBD = Database.instance();
  }

  /*
   * Lista categorias do sistema.
   * @return ArrayList
   */
  public ArrayList listarCategorias() throws SIAlocacaoException {

    Connection con = null;
    try {
      con = poolBD.getConnection();
      PreparedStatement cmd =
          con.prepareStatement("SELECT * FROM categoria");

      ResultSet result = cmd.executeQuery();

      ArrayList categorias =
          new ArrayList();

      while (result.next()) {
        categorias.add(
                  new Categoria(result.getInt("codigo"),
                     result.getString("descricao") ));
      }
      result.close();
      cmd.close();
      return categorias;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
        try {
          poolBD.returnConnection(con);
        } catch (Exception e) {
          throw new SIAlocacaoException(e.getMessage(), e);
        }
    }
  }
}