package sialocacao.sala.excecoes;

/**
 * <p>Title: SalaInexistenteException.java </p>
 * <p>Description: Responsvel por tratar exceo de sala inexistente. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import sialocacao.*;
import sialocacao.sala.*;

public class CategoriaInexistenteException extends SIAlocacaoException {

  /*
   * Construtor responsvel por definir o cdigo da Categoria inexistente.
   */
  public CategoriaInexistenteException(int codigo) {
    setDescription(getDescription(codigo));
  }

  /*
   * Construtor responsvel por definir a descricao da sala inexistente.
   */
  public CategoriaInexistenteException(String descricao) {
    setDescription(getDescription(descricao));
  }

  /*
   * Construtor responsvel por definir a sala inexistente.
   */
  public CategoriaInexistenteException(Categoria categoria) throws
      IllegalArgumentException {

    if (categoria == null) {
      throw new IllegalArgumentException("categoria == null");
    }

    setDescription(getDescription(categoria.getCodigo()));
  }

  /*
   * Define descrio da exceo, passando o cdigo da categoria inexistente.
   * @param codigo
   */
  private String getDescription(int codigo) {
    StringBuffer description =
        new StringBuffer("Nao existe categoria com o codigo ");

    description.append(codigo);

    return description.toString();
  }

  /*
   * Define descrio da exceo, passando a descricao da categoria inexistente.
   * @param descricao
   */
  private String getDescription(String descricao) {
    StringBuffer description =
        new StringBuffer("Nao existe categoria com a descricao ");

    description.append(descricao);

    return description.toString();
  }
}
