package sialocacao.sala.negocio;

/**
 * <p>Title: CadastroSalas.java </p>
 * <p>Description: Implementa a camanda de negcio de salas. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.util.*;

import sialocacao.*;
import sialocacao.sala.*;
import sialocacao.sala.database.*;
import sialocacao.sala.excecoes.*;

public class CadastroSalas implements ICadastroSalas {

  /*
   * Instncia do repositrio de salas.
   */
  private ISalaDatabase salaDatabase;

  /*
   * Construtor default definindo o repositrio de salas.
   *
   */
  public CadastroSalas() throws Exception{
    setSalaDatabase(new SalaDatabase());
  }

  /*
   * Construtor definindo o repositrio de salas.
   * @param salaDatabase
   */
  public CadastroSalas(ISalaDatabase salaDatabase) {
    setSalaDatabase(salaDatabase);
  }

  /*
   * Define repositrio de salas.
   * @param salaDatabase
   */
  public void setSalaDatabase(ISalaDatabase salaDatabase) throws
      IllegalArgumentException {

    if (salaDatabase == null) {
      throw new IllegalAccessError("salaDatabase == null");
    }

    this.salaDatabase = salaDatabase;
  }

  /*
   * Insere sala no sistema.
   * @param descricao
   * @param localizacao
   * @param observacao
   */
  public void inserirSala(String descricao, String localizacao, String observacao,
    int codigoCategoria)
       throws SIAlocacaoException {

    this.salaDatabase.inserirSala(descricao, localizacao, observacao,
      codigoCategoria);
  }

  /*
   * Altera sala no sistema.
   * @param sala
   */
  public void alterarSala(Sala sala) throws SIAlocacaoException,
      SalaInexistenteException {

    this.salaDatabase.alterarSala(sala);
  }

  /*
   * Exclui sala do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirSala(int codigo) throws SIAlocacaoException,
      SalaInexistenteException {

    this.salaDatabase.excluirSala(codigo);
  }

  /*
   * Busca sala no sistema pelo cdigo.
   * @param codigo
   * @return Sala
   */
  public Sala buscarSala(int codigo) throws SIAlocacaoException,
      SalaInexistenteException {

    return this.salaDatabase.buscarSala(codigo);
  }

  /*
   * Busca sala no sistema pelo cdigo.
   * @param descricao
   * @return Sala
   */
  public Sala buscarSala(String descricao) throws SIAlocacaoException,
      SalaInexistenteException {

    return this.salaDatabase.buscarSala(descricao);
  }

  /*
   * Lista salas do sistema.
   * @return ArrayList
   */
  public ArrayList listarSalas() throws SIAlocacaoException {

    return this.salaDatabase.listarSalas();
  }
}