package sialocacao.timetabling;

/**
 * <p>Title: AlgoritmoTimetabling.java </p>
 * <p>Description: Interface para um algoritmo de timetabling que fornecer uma sugesto de Alocacao.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.util.*;

public interface AlgoritmoTimetabling {

	/**
	 * Mtodo que prope uma sugesto de alocao, levando em considerao as prioridades dos
	 * professores com relao aos horrios e a adequao da sala em que as disciplinas sero
	 * ministradas.
	 * @param disciplinasProfessores Relao identificando as associaes disciplina-professor que
	 * devem ser consideradas. Armazenar no Map: key = codigoDisciplina, value = codigoProfessor
	 * @param preferenciasProfessores Relao da disponibilidade do professor de acordo com os
	 * horrios. Armazenar no Map: key = codigoProfessor, value = Map(codigoHorario -> prioridade)
	 * @param adequacaoDisciplinasSalas Relao com adequao de categorias de salas a disciplinas.
	 * Armazenar no Map: key = codigoDisciplina, value = Map(codigoCategoria -> adequao)
	 * @param salasCategorias Relao identificando a que categoria uma determinada sala
	 * pertence. Armazenar no Map: key = codigoSala, value = codigoCategoria
	 * @param aulasDisciplina Nmero de aulas por disciplina. Armazenar no Map:
	 * key = codigoDisciplina, value = numeroDeAulas.
	 * @param diasUteis Quantidade de dias uteis que deve ser considerada.
	 * @param salas Conjunto de salas disponveis para alocao.
	 * @param horarios Conjunto de horrios passveis de alocao
	 * @param professores Conjunto de professores existentes.
	 * @param disciplinas Conjunto de disciplinas existentes.
	 * @return Uma coleo com todos os elementos da alocao proposta.
	 */
	public Collection proporAlocacao(Map disciplinasProfessores, Map preferenciasProfessores,
			  Map adequacaoDisciplinasSalas, Map salasCategorias, Map aulasDisciplinas,
			  int diasUteis, List salas, List horarios, List professores, List disciplinas);

}

/*
------=_20030822023529_83559
Content-Type: text/plain; name="Horario.java"
Content-Disposition: attachment; filename="Horario.java"

package sialocacao.alocacao;

/**
 * <p>Title: Alocacao.java </p>
 * <p>Description: Responsvel por manipular o objeto alocacao. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 2.0
 */
/*
import java.util.*;

public class Horario implements Comparable {


  public static final String SEGUNDA = "Segunda";

  public static final String TERCA = "Tera";

  public static final String QUARTA = "Quarta";

  public static final String QUINTA = "Quinta";

  public static final String SEXTA = "Sexta";

  public static final String SABADO = "Sbado";

  /*
   * Cdigo do horario.
   *
  private int codigo;

  /*
   * Dia da semana do horario.
   *
  private String diaDaSemana;

  /*
   * Hora inicio de horario.
   *  private String horaInicio;

  /*
   * Hora final de horario.
   *
  private String horaFim;

  /*
   * Construtor vazio.
   *
  public Horario() {
  }

  /*
   * Construtor
   *
  public Horario(int codigo, String diaDaSemana, String horaInicio, String horaFim) {
    setCodigo(codigo);
    setDiaDaSemana(diaDaSemana);
    setHoraInicio(horaInicio);
    setHoraFim(horaFim);
  }

  /**
   * Pega cdigo do horario.
   * @return int
   *
  public int getCodigo() {
    return this.codigo;
  }

  /**
   * Define cdigo do horario.
   * @param codigo
   *
  public void setCodigo(int codigo) {
    this.codigo = codigo;
  }

  /**
   * Pega dia da semana do horario.
   * @return String
   *
  public String getDiaDaSemana() {
    return this.diaDaSemana;
  }

  /**
   * Define dia da semana do horario.
   * @param diaDaSemana
   *
  public void setDiaDaSemana(String diaDaSemana) {
    this.diaDaSemana = diaDaSemana;
  }

  /**
   * Pega hora de incio do horario.
   * @return String
   *
  public String getHoraInicio() {
    return this.horaInicio;
  }

  /**
   * Define hora final do horario.
   * @param horaFim
   *
  public void setHoraInicio(String horaInicio) {
    this.horaInicio = horaInicio;
  }

  /**
   * Define hora final do horario.
   * @param horaFim
   *
  public void setHoraFim(String horaFim) {
    this.horaFim = horaFim;
  }

  /**
   * Pega hora final do horario.
   * @return String
   *
  public String getHoraFim() {
    return this.horaFim;
  }

  /**
   * Compara dois objetos horario.
   * @param horario
   * @return boolean
   *
  public boolean equals(Object horarioObjeto) {
    boolean result = false;
    Horario horario = (Horario)horarioObjeto;
    if((this.horaInicio.equals(horario.getHoraInicio())) &&
       (this.horaFim.equals(horario.getHoraFim())) )

      result = true;

    return result;
  }

  public int hashCode() {
    return 1;
  }

  /**
   * Compara dois objetos horario, leva em considerao o dia da semana e a hora de inicio.
   * @return Um nmero negativo se o this for menor, zero se os objetos forem iguais e um
   * nmero positivo se o objeto this for maior.
   *
  public int compareTo(Object objeto) {
    Horario horario = (Horario)objeto;
    HashMap diasSemana = new HashMap();
    diasSemana.put(SEGUNDA, new Integer(1));
	diasSemana.put(TERCA, new Integer(2));
	diasSemana.put(QUARTA, new Integer(3));
	diasSemana.put(QUINTA, new Integer(4));
	diasSemana.put(SEXTA, new Integer(5));
	diasSemana.put(SABADO, new Integer(6));

	int diferencaDias = ((Integer)diasSemana.get(this.diaDaSemana)).intValue()
			- ((Integer)diasSemana.get(horario.getDiaDaSemana())).intValue();

	int diferencaHoras = Integer.parseInt(this.horaInicio.substring(0, this.horaInicio.indexOf(":")))
			- Integer.parseInt(horario.getHoraInicio().substring(0, horario.getHoraInicio().indexOf(":")));

	int diferencaMinutos = Integer.parseInt(this.horaInicio.substring(this.horaInicio.indexOf(":")+1))
			- Integer.parseInt(horario.getHoraInicio().substring(horario.getHoraInicio().indexOf(":")+1));

	if (diferencaDias == 0){
	  if (diferencaHoras == 0)
	    return diferencaMinutos;
	  else
	  	return diferencaHoras;
	} else {
	  return diferencaDias;
	}
  }

  /**
   * Concatena atributos do objeto horario em uma string.
   * @return String
   *
  public String toString() {
    StringBuffer str = new StringBuffer();

    str.append("sialocacao.alocacao.Horario: (");
    str.append(this.codigo);
    str.append(", ");
    str.append(this.diaDaSemana);
    str.append(", ");
    str.append(this.horaInicio);
    str.append(", ");
    str.append(this.horaFim);

    str.append(")");

    return str.toString();
  }


}
*/