package sialocacao.util;

/**
 * <p>Title: Data.java </p>
 * <p>Description: Manipula datas no formato dd/mm/aaaa.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Carlos Jnior
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.util.*;

public class Data {

  private int dia, mes, ano;

  public static final int JANEIRO = 1;
  public static final int FEVEREIRO = 2;
  public static final int MARCO = 3;
  public static final int ABRIL = 4;
  public static final int MAIO = 5;
  public static final int JUNHO = 6;
  public static final int JULHO = 7;
  public static final int AGOSTO = 8;
  public static final int SETEMBRO = 9;
  public static final int OUTUBRO = 10;
  public static final int NOVEMBRO = 11;
  public static final int DEZEMBRO = 12;

  // contm o nmero de dias de cada ms
  // colocou-se 0 no primeiro parmetro para se facilitar a leitura do cdigo
  // nos mtodos abaixo
  private int [] numDiasMes = {0,31,28,31,30,31,30,31,31,30,31,30,31};

/**
  Cria data atual
  */
  public Data() {
    Calendar meuCalendario = new GregorianCalendar();
    this.dia = meuCalendario.get(Calendar.DAY_OF_MONTH);
    this.mes = meuCalendario.get(Calendar.MONTH)+1;
    this.ano = meuCalendario.get(Calendar.YEAR);
  }

/**
 * @param dia  o dia
 * @param mes  o mes
 * @param ano  o ano
 */
  public Data(int dia, int mes, int ano) {
    this.dia = dia;
    this.mes = mes;
    this.ano = ano;
  }

  /**
  * @param dataString Data no formato String
  */
  public Data(String dataString) {
    String barras = "";
    if(! (dataString.substring(2,3)).equals("/")){
      dataString = "0"+dataString;
    } if(! (dataString.substring(5,6)).equals("/")){
      dataString = dataString.substring(0,3)+"0"+dataString.substring(3,9);
    }
    try {
      barras = dataString.substring(2,3) + dataString.substring(5,6);
      this.dia = Integer.parseInt(dataString.substring(0, 2));
      this.mes = Integer.parseInt(dataString.substring(3, 5));
      this.ano = Integer.parseInt(dataString.substring(6, 10));
    } catch (NumberFormatException e) {
      new Data(0,0,0);
    } catch (IndexOutOfBoundsException e) {
      new Data(0,0,0);
    }
    if (!"//".equals(barras))
      new Data(0,0,0);
  }

  /**
  * Este mtodo retorna o ano da data
  * @return o ano da data
  */
  public int getAno() {
    return ano;
  }

  /**
  * Este mtodo retorna o ms da data
  * @return o ms da data
  */
  public int getMes() {
    return mes;
  }

  /**
  * Este mtodo retorna o dia da data
  * @return o dia da data
  */
  public int getDia() {
    return dia;
  }

  /**
  * Este mtodo altera o ano da data
  * @param ano  o novo ano da data
  */
  public void setAno(int ano) {
    this.ano = ano;
  }

  /**
  * Este mtodo altera o dia da data
  * @param dia  o novo dia da data
  */
  public void setDia(int dia) {
    this.dia = dia;
  }

  /**
  * Este mtodo altera o ms da data
  * @param mes  o novo ms da data
  */
  public void setMes(int mes) {
    this.mes = mes;
  }

  /**
  * Este mtodo indica se uma data  menor do que outra
  * @param data  a data a ser comparada com this
  * @return indicando se a data this  anterior ao do argumento
  */
  public boolean menorQue(Data data) {
    if(this.anoMenor(data))
      return true;
    else {
      if(this.isMesmoAno(data)) {
        if(this.mesMenor(data))
	  return true;
        else {
	  if(this.isMesmoMes(data))
	    if(this.diaMenor(data))
	      return true;
          return false;
        }
      }
      return false;
    }
  }

  /**
  * Este mtodo indica se o dia de this e de data sao iguais
  * @para data  a data a ser comparada
  * @return indicando se eles possuem o mesmo dia
  */
  private boolean isMesmoDia(Data data) {
    return this.dia == data.getDia();
  }

  /**
  * Este mtodo indica se o mes de this e de data sao iguais
  * @para data  a data a ser comparada
  * @return indicando se eles possuem o mesmo mes
  */
  private boolean isMesmoMes(Data data) {
    return this.mes == data.getMes();
  }

  /**
  * Este mtodo indica se o ano de this e de data sao iguais
  * @para data  a data a ser comparada
  * @return indicando se eles possuem o mesmo ano
  */
  private boolean isMesmoAno(Data data) {
    return this.ano == data.getAno();
  }

  /**
  * Este mtodo indica se o dia do this  menor do que o do parmetro
  * @param data  a data a ser comparada
  * @return indicando se a o dia do this  menor ou no do que de data
  */
  private boolean diaMenor(Data data) {
    return this.getDia()<data.getDia();
  }

  /**
  * Este mtodo indica se o mes do this  menor do que o do parmetro
  * @param data  a data a ser comparada
  * @return indicando se a o mes do this  menor ou no do que de data
  */
  private boolean mesMenor(Data data) {
    return this.getMes()<data.getMes();
  }

  /**
  * Este mtodo indica se o ano do this  menor do que o do parmetro
  * @param data  a data a ser comparada
  * @return indicando se a o ano do this  menor ou no do que de data
  */
  private boolean anoMenor(Data data) {
    return this.getAno()<data.getAno();
  }

  /**
  * Este mtodo indica se o ano da data  bissexto ou no
  * @return se o ano  bissexto ou no
  */
  public boolean isBissexto() {
    if(isMultiplo(this.ano,4))
      return true;
    return false;
  }

  /**
  * Este mtodo indica se num  multiplo de mult
  * @return indicando se num  multiplo de mult
  */
  private static boolean isMultiplo(int num,int mult) {
    return (num%mult==0);
  }

  /**
  * Este mtodo retorna um conjunto de datas teis entre inicio e fim que obedecem a uma determinada condio.
  * @return um conjunto de datas teis entre inicio e fim que satisfazem uma restrio.
  */
  public static Vector getDiasUteis(Data inicio, Data fim, Vector diasUteis) {
    Vector retorno = new Vector();
    while(inicio.menorQue(fim)) {
      Calendar data = new GregorianCalendar(inicio.getAno(), inicio.getMes() - 1, inicio.getDia());
      if (diasUteis.contains(new Integer(data.get(Calendar.DAY_OF_WEEK))))
        retorno.add(inicio);
      inicio = inicio.getProximaData(1);
    }
    return retorno;
  }

  /**
  * Este mtodo retorna a prxima data somado somaDias a data this
  * @param somaDias  o nmero de dias na frente da data this
  * @return a prxima data somando somaDias a data this
  */
  public Data getProximaData(int somaDias) {
    this.verificaAnoBissexto();
    int dia = this.getNextDay(somaDias);
    int mes = this.getMonth(somaDias);
    int ano = this.getYear(mes);
    return new Data(dia,mes,ano);
  }

  /**
  * Este mtodo indica se mudou de ano ao se somar a data
  * @param mes  o mes da nova data
  * @return indicando se mudou ou nao de ano
  */
  private boolean mudouDeAno(int mes) {
    return (this.mes==12 && mes==1);
  }

  /**
  * Este mtodo retorna o prximo dia somando somaDias
  * @param somaDias  o nmero de dias na frete da data this
  * @return o prximo dia somando somaDias
  */
  private int getNextDay(int somaDias) {
    int dias = this.dia+somaDias;
    if(dias > 0) {
      int dia = (this.dia + somaDias) % this.numDiasMes[this.mes];
      if (dia == 0)
        dia = this.numDiasMes[this.mes];
        return dia;
    } else {
      int mes = this.mes-1;
      if(mes==0) mes = 12;
      int dia = this.numDiasMes[mes] + (dias);
      return dia;
    }
  }

  /**
  * Este mtodo retorna o ms quando se soma somaDias a data corrente
  * @param somaDias  o nmero de dias na frete da data this
  * @return o ms quando se soma somaDias
  */
  private int getMonth(int somaDias) {
    int mes = this.mes;
    if((this.dia+somaDias)>this.numDiasMes[this.mes])
      mes = (this.mes+1)%12;
    if((this.dia+somaDias)<=0)
      mes = this.mes-1;
    if(mes==0)
        mes=12;
    return mes;
  }

  /**
  * Este mtodo indica o ano a partir do novo mes
  * @param mes  o novo mes
  * @return o ano a partir de um novo mes obtido pela soma de dias
  */
  private int getYear(int mes) {
    int ano = this.ano;
    if(mudouDeAno(mes))
      ano++;
    if(this.mes==1 && mes==12)
      ano--;
    return ano;
  }

  /**
  * Este mtodo verifica se um ano  bissexto e faz o tratamento caso seja.
  */
  private void verificaAnoBissexto() {
    if(this.isBissexto())
      this.numDiasMes[2] = 29;
    else
      this.numDiasMes[2] = 28;
  }

  /**
  * Este mtodo compara 2 datas
  * @param data  a data a ser comparada
  * @return se as datas sao iguais
  */
  public boolean equals(Object obj) {
    Data data = (Data)obj;
    if(this.isMesmoAno(data))
      if(this.isMesmoMes(data))
        if(this.isMesmoDia(data))
	  return true;
    return false;
  }

  /**
  * Este mtodo  o toString da classe
  * @return um String contendo os dados do objeto
  */
  public String toString() {
    return this.dia+"/"+this.mes+"/"+this.ano;
  }

  /**
  * Este mtodo testa se uma data  vlida
  * @return indicando se esta data  vlida
  */
  public boolean isValida() {
    this.verificaAnoBissexto();
    if(this.isNumerosPositivos())
      if(this.isNumeroDeDiasOk())
        return true;
    return false;
  }

  /**
  * Este mtodo indica se o dia, mes e ano sao positivos
  * @return se sao positivos
  */
  private boolean isNumerosPositivos()  {
    return this.ano>0 && this.mes>0 && this.dia>0;
  }

  /**
  * Este mtodo indica se o ms possui o nmero de dias
  * @return se possui este nmero de dias
  */
  private boolean isNumeroDeDiasOk() {
    return this.dia<=this.numDiasMes[this.mes];
  }

  /**
  * Mtodo para somar anos a uma data
  *@param somaAnos  o nmero de anos na frente da data this
  * @return a prxima data somando somaAnos a data this
  */
  public Data somaAnos(int somaAnos) {
    int ano = this.ano+somaAnos;
    Data d = new Data(this.dia, this.mes, ano);
    if(d.isBissexto() && this.dia == 29 && this.mes == 2){
      this.dia = 28;
    }
    return new Data(dia,mes,ano);
  }
}